#!/usr/bin/python -tt

import getpass

import genshi.template
import psycopg2

#
# FAS queries
#

# Users in danger of having account permissions revoked: those that are in
# cla_done only because they've signed cla_fedora
# Template: fas_users
# Input: None
# Output: fas_users

QUERY_TEMPLATES = {'fas_users':
'''
select username from people as p, person_roles as r, groups as g
-- join tables
where p.id = r.person_id and g.id = r.group_id
-- in cla_done now
and g.name = 'cla_done' and r.role_status = 'approved'
-- are not in a non-cla_fedora/cla_done group
and p.username not in (
  select username from people as p, person_roles as r, groups as g
  where p.id = r.person_id and g.id = r.group_id
  -- in a cla group
  and g.group_type = 'cla' and r.role_status = 'approved'
  -- that's not cla_fedora or cla_done
  and g.name not in ('cla_fedora', 'cla_done'))
order by username
''',

# What groups require cla_fedora or cla_done as a prerequisite?
# Template: cla_req_groups
# Input: None
# Output: cla_req_groups

'cla_req_groups':
'''select name from groups
where prerequisite_id in (
  select id from groups where name in ('cla_done', 'cla_fedora')
  )
order by name
''',

# What fedorahosted groups require cla_* as a prereq?
# Template: cla_req_hosted_groups
# Input: None
# Output: cla_req_hosted_groups

'cla_req_hosted_groups':
'''select name from groups
where prerequisite_id in (
  select id from groups where name in ('cla_done', 'cla_fedora')
  )
and group_type in ('svn', 'bzr', 'git', 'hg')
and name not in ('packager', 'cvsadmin)
order by name
''',

# What users in danger of losing permissions are in a cla_* requiring group?
# Template: cla_req_query
# Input: {'groups': cla_req_groups, 'output_column': 'p.username',
# 'users': fas_users}
# Output: users_in_cla_req_groups

'cla_req_query':
'''select ${output_column} from groups as g, person_roles as r, people as p
where g.id = r.group_id and p.id = r.person_id
and g.name in (
${', '.join(["'%s' % g for g in groups])}
)
and r.role_status = 'approved'
and p.username in (
${', '.join(["'%s'" % u for u in users])}
)
order by p.username
''',

# What users in danger of losing permissions are in a cla_* requiring
# fedorahosted group?
# Template: cla_req_query
# Input: {'groups': cla_req_hosted_groups, 'output_column': 'p.username',
# 'users': fas_users}
# Output: users_in_cla_req_hosted_groups

# What groups that require cla_* as a prereq have people 
# Template: cla_req_query
# Input: {'groups': cla_req_groups, 'output_column': 'g.name',
# 'users': fas_users}
# Output: groups that are in danger of losing members
 
# What users in danger of losing permissions are in a cla_* requiring
# fedorahosted group?
# Template: cla_req_query
# Input: {'groups': cla_req_hosted_groups, 'output_column': 'g.name',
# 'users': fas_users}
# Output: groups that are in danger of losing members

#
# PackageDB ownership
#

# Which users in danger of losing account perms are owners of a package?
# Template: package_ownership
# Input: {'output_column': 'pl.owner', 'users': fas_users})
# Ouptut: package_owners

'package_ownership':
'''
select distinct ${output_column} from packagelisting as pl, package as p, collection as c
where pl.packageid = p.id and pl.collectionid = c.id
-- statuscodes for releases that are in development or active
and c.statuscode in (18, 1)
-- statuscodes for packages that are Approved
and p.statuscode in (3)
-- statuscode for packagelistings that are Approved
and pl.statuscode in (3);
and pl.owner in (
${', '.join(["'%s'" % u for u in users])}
)
order by ${output_column}
''',

# Which packages have owners that are in danger of losing account perms?
# Template: package_ownership
# Input: {'output_column': 'p.name', 'users': fas_users}
# Output: packages_losing_owner

# Which users in danger of losing perms are comaintainers or committers of
# a package?
# Template: package_acl_check
# Input: {'output_column': 'ppl.username', 'users': fas_users, acls:
# ['commit', 'approveacls']}
# Output: packages_comaint

'package_acl_check':
'''select distinct ${output_column} from packagelisting as pl, package as p, collection as c,
  personpackagelisting as ppl, personpackagelistingacl as ppla
-- join tables
where p.id = pl.packageid and c.id = pl.collectionid
and pl.id = ppl.packagelistingid and ppl.id = ppla.personpackagelistingid
-- collection is active
and c.statuscode in (18, 1)
-- package and packagelisting are Approved
and p.statuscode in (3) and pl.statuscode in (3)
-- user has an acl on a package
and ppl.username in (
${', '.join(["'%s'" % u for u in users])}
)
-- that acl is an approved commit or approveacls
and ppla.acl in (
${', '.join(["'%s'" % a for a in acls])}
) and ppla.statuscode in (3)
order by ${output_column}
''',

# Which packages have comaintainers or committers that are in danger of losing
# permissions?
# Template: package_acl_check
# Input: {'output_column': 'p.name', 'users': fas_users, acls: ['commit',
# 'approveacls']}
# Output: packages_losing_comaint

# Which watchers of a package are potentially going to lose permissions?
# Template: package_acl_check
# Input: {'output_column': 'ppl.username', 'users': fas_users, acls: ['watchcommits',
# 'watchbugzilla']}
# Output: package_watchers

# Which packages have watchers that are potentially going to lose permissions?
# Template: package_acl_check
# Input: {'output_column': 'p.name', 'users': fas_users, acls: ['watchcommits',
# 'watchbugzilla']}
# Output: packages_losing_watchers

}

def write_log(filename, lines):
    f = open(filename, 'w')
    f.write('\n'.join(lines))
    f.close()

def fas_queries(fas_query, fas_users):

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['cla_req_groups'])

    query_string = query_template.generate()
    cla_req_groups = fas_query.execute(query_string).fetchall()
    write_log('cla_req_groups.txt', cla_req_groups)

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['cla_req_hosted_groups'])

    query_string = query_template.generate()
    cla_req_hosted_groups = fas_query.execute(query_string).fetchall()
    write_log('cla_req_hosted_groups.txt', cla_req_hosted_groups)

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['cla_req_query'])

    query_string = query_template.generate(groups=cla_req_groups, output_column='p.username',
        users=fas_users)
    users_in_cla_req_groups = fas_query.execute(query_string).fetchall()
    write_log('users_in_cla_req_groups.txt', users_in_cla_req_groups)

    query_string = query_template.generate(groups=cla_req_hosted_groups,
            output_column='p.username', users=fas_users)
    users_in_cla_req_hosted_groups = fas_query.execute(query_string).fetchall()
    write_log('users_in_cla_req_hosted_groups.txt', users_in_cla_req_hosted_groups)

    query_string = query_template.generate(groups=cla_req_groups, output_column='g.name',
        users=fas_users)
    groups_losing_members = fas_query.execute(query_string).fetchall()
    write_log('groups_losing_members.txt', groups_losing_members)
 
    query_string = query_template.generate(groups=cla_req_hosted_groups, output_column='g.name',
        users=fas_users)
    hosted_groups_losing_members = fas_query.execute(query_string).fetchall()
    write_log('hosted_groups_losing_members.txt', hosted_groups_losing_members)

def pkgdb_queries(pkgdb_query, fas_users):

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['package_ownership'])
    query_string = query_template.generate(output_column='pl.owner', users=fas_users)
    package_owners = pkgdb_query.execute(query_string).fetchall()
    write_log('package_owners.txt', package_owners)

    query_string = query_template.generate(output_column='p.name', users=fas_users)
    packages_losing_owner = pkgdb_query.execute(query_string).fetchall()
    write_log('packages_losing_owner.txt', packages_losing_owner)

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['package_acl_check'])
    query_string = query_template.generate(output_column='ppl.username',
            acls=['commit', 'approveacls'], users=fas_users)
    packages_comaint = pkgdb_query.execute(query_string).fetchall()
    write_log('packages_comaint.txt', packages_comaint)

    query_string = query_template.generate(output_column='p.name',
            acls=['commit', 'approveacls'], users=fas_users)
    packages_losing_comaint = pkgdb_query.execute(query_string).fetchall()
    write_log('packages_losing_comaint.txt', packages_losing_comaint)

    query_string = query_template.generate(output_column='ppl.username',
            acls=['watchcommits', 'watchbugzilla'], users=fas_users)
    package_watchers = pkgdb_query.execute(query_string).fetchall()
    write_log('package_watchers.txt', package_watchers)

    query_string = query_template.generate(output_column='p.name',
            acls=['watchcommits', 'watchbugzilla'], users=fas_users)
    packages_losing_watchers = pkgdb_query.execute(query_string).fetchall()
    write_log('packages_losing_watchers.txt', packages_losing_watchers)

def main():
    username = raw_input('FAS DB username: ')
    password = getpass.getpass('FAS DB Password: ')
    fas_db = psycopg2.connect(database='fas2', host='127.0.0.1', user=username, password=password)
    fas_query = fas_db.cursor()

    query_template = genshi.templates.TextTemplate(QUERY_TEMPLATES['fas_users'])
    query_string = query_template.generate()
    fas_users = fas_query.execute(query_string).fetchall()
    write_log('fas_users.txt', fas_users)

    fas_queries(fas_query, fas_users)
    fas_db.close()

    username = raw_input('PKGDB DB username: ')
    password = getpass.getpass('PKGDB DB Password: ')
    pkgdb_db = psycopg2.connect(database='pkgdb', host='127.0.0.1', user=username, passsword=password)
    pkgdb_query = pkgdb_db.cursor()
    pkgdb_queries(pkgdb_query, fas_users)
    pkgdb_db.close()


if __name__ == '__main__':
    main()
